--[[ ---------------------------------------------------------------------------
Name:	SimpleRaidFrameOverlays
Author:	Gallowbraid of Terokkar
About:	A very simple enhancement of the built-in raid frames that allows
        overlay colors to be displayed on raid frames for units that have 
        any sort of debuff. This mimics the functionality of almost all raid
        frame addons without much overhead.
----------------------------------------------------------------------------- ]]

local SRFO = LibStub("AceAddon-3.0"):NewAddon("SimpleRaidFrameOverlays", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SimpleRaidFrameOverlays")

SRFO.defaults = {
	global = {
		testmode = false,
		testframewidth = 200,
		testframeheight = 100,
	},
	profile = {},
}

local UnitFrames = {}
local OverlayFrames = {}
local OrphanedFrames = {}
local TestFrames = {}

SRFO.options = {
	type = 'group',
	name = L["SimpleRaidFrameOverlays"],
	handler = SRFO,
	args = {
		general = {
			type = 'group',
			name = L["General"],
			args = {
				testmode = {
					type = 'toggle',
					name = L["TestMode"],
					desc = L["TestModeDesc"],
					get = function(info) return info.handler.db.global.testmode end,
					set = function(info, val)
						info.handler.db.global.testmode = val
						SRFO:TestingModeChanged()
					end,
					order = 1,
				},
				testframewidth = {
					type = 'range',
					name = L["TestFrameWidth"],
					desc = L["TestFrameWidthDesc"],
					min = 50,
					max = 500,
					softMin = 50,
					softMax = 500,
					step = 1,
					bigStep = 1,
					get = function(info) return info.handler.db.global.testframewidth end,
					set = function(info, val)
						info.handler.db.global.testframewidth = val
						SRFO:TestingModeChanged()
					end,
					width = "full",
					order = 2,
				},
				testframeheight = {
					type = 'range',
					name = L["TestFrameHeight"],
					desc = L["TestFrameHeightDesc"],
					min = 20,
					max = 400,
					softMin = 20,
					softMax = 400,
					step = 1,
					bigStep = 1,
					get = function(info) return info.handler.db.global.testframeheight end,
					set = function(info, val)
						info.handler.db.global.testframeheight = val
						SRFO:TestingModeChanged()
					end,
					width = "full",
					order = 3,
				},
			},
		},
	},
}

function SRFO:OnInitialize()
	-- Set up the SavedVariables DB
	self.db = LibStub("AceDB-3.0"):New("SimpleRaidFrameOverlaysDB", self.defaults)
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profiles.order = 1000
	
	-- Set up the InterfaceOptions
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("SimpleRaidFrameOverlays", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleRaidFrameOverlays", "SimpleRaidFrameOverlays")
	
	-- Register slash commands
	self:RegisterChatCommand("simpleraidframeoverlays", function() InterfaceOptionsFrame_OpenToCategory("SimpleRaidFrameOverlays") end)
	self:RegisterChatCommand("srfo", function() InterfaceOptionsFrame_OpenToCategory("SimpleRaidFrameOverlays") end)
end

-- Registers all of the events used by this addon.
function SRFO:OnEnable()
	self:RegisterEvent("PARTY_CONVERTED_TO_RAID", "UnitFramesChanged")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UnitFramesChanged")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "UnitFramesChanged")
	-- Enable testing if possible.
	self:TestingModeChanged()
end

-- Unregisters all of the events used by this addon.
function SRFO:OnDisable()
	self:UnregisterEvent("PARTY_CONVERTED_TO_RAID")
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	self:UnregisterEvent("RAID_ROSTER_UPDATE")
end

-- Cleans the cache of the unit frames due to the changing raid or party frames.
function SRFO:UnitFramesChanged()
	for name, module in SRFO:IterateModules() do
		if(module:IsEnabled()) then
			module:CleanFrames()
			module:UpdateAddonStatus()
		end
	end
	UnitFrames = {}
end

-- Updates the testing frame to reflect the testing mode.
function SRFO:TestingModeChanged()
	-- Hide or show the frame depending on the option.
	if(SRFO.db.global.testmode) then
		for i=1,4 do
			TestFrames[i]:SetWidth(SRFO.db.global.testframewidth)
			TestFrames[i]:SetHeight(SRFO.db.global.testframeheight)
			TestFrames[i]:SetPoint("RIGHT", UIParent, "RIGHT", 0.0, -SRFO.db.global.testframeheight*(i-1))
			TestFrames[i]:Show()
		end
		-- Tell all the modules to update their testing stuff.
		for name, module in SRFO:IterateModules() do
			if(module:IsEnabled()) then module:UpdateForTesting() end
		end
	else
		for k,v in pairs(TestFrames) do
			v:Hide()
		end
	end
end

-- Indicates whether we're in testing mode.
function SRFO:InTestingMode()
	return self.db.global.testmode
end

-- Checks to see if the unit is in the player's party.
-- @param unit
--   The name of the unit to test.
-- @return
--   true if the unit is "partyN" or "player" if the player is in a party.
function SRFO:IsPartyUnit(unit)
	return strmatch(unit, "^party%d+$") or (unit == "player" and GetNumPartyMembers() > 0) or unit == "testparty"
end

-- Checks to see if the unit is a party pet.
-- @param unit
--   The name of the unit to test.
-- @return
--   true if the unit is "partypetN".
function SRFO:IsPartyPetUnit(unit)
	return strmatch(unit, "^partypet%d+$") or (unit == "pet" and GetNumPartyMembers() > 0) or unit == "testpartypet"
end

-- Checks to see if the unit is a raid member.
-- @param unit
--   The name of the unit to test.
-- @return
--   true if the unit is "raidN".
function SRFO:IsRaidUnit(unit)
	return strmatch(unit, "^raid%d$") or unit == "testraid"
end

-- Checks to see if the unit is a raid pet.
-- @param unit
--   The name of the unit to test.
-- @return
--   true if the unit is "raidpetN".
function SRFO:IsRaidPetUnit(unit)
	return strmatch(unit, "^raidpet%d$") or unit == "testraidpet"
end

-- Retrieves the frame for the given unit.
-- @param unit
--   The name of the unit whose frame to retrieve
-- @param unitType
--   The type of the unit:
--    - 1: party player
--    - 2: party pet
--    - 3: raid player
--    - 4: raid pet
-- @return
--   The unit frame for that unit.
function SRFO:GetUnitFrame(unit, unitType)
	-- If the unit frame has not already been found, search for it using the type.
	if(self.db.global.testmode) then
		return TestFrames[unitType]
	else
		if(not UnitFrames[unit]) then
			if(unitType == 1) then
				
				-- Party frames supported are the "Compact" raid frames for party implemented by blizzard.
				-- See if any of these frames match the unit.
				for i=1, 5 do
					local f = getglobal("CompactPartyFrameMember" .. tostring(i))
					if(f) then
						local fu = f:GetAttribute("unit")
						if(UnitIsUnit(fu, unit)) then
							UnitFrames[unit] = f
							break
						end
					else
						break
					end
				end
			elseif(unitType == 2) then
				-- not sure what the frames for these are just yet as no pets are shown in raid frame mode.
			elseif(unitType == 3) then
				-- If the units are grouped, we can loop through by group then by member of group to see if
				-- the frames match the unit.
				for i=1, 5 do
					for j=1, 5 do
						local f = getglobal("CompactRaidGroup" .. tostring(i) .. "Member" .. tostring(j))
						if(f) then
							local fu = f:GetAttribute("unit")
							if(fu) then
								if(UnitIsUnit(fu, unit)) then
									UnitFrames[unit] = f
									break
								end
							end
						end
					end
				end
				-- If the units are not grouped, then look through all of the compact frames.
				if(not UnitFrames[unit]) then
					for i=1, 60 do
						local f = getglobal("CompactRaidFrame" .. tostring(i))
						if(f) then
							local fu = f:GetAttribute("unit")
							if(UnitIsUnit(fu, unit)) then
								UnitFrames[unit] = f
								break
							end
						end
					end
				end
			elseif(unitType == 4) then
				-- Raid pets are always in the compact raid frames.
				for i=1, 60 do
					local f = getglobal("CompactRaidFrame" .. tostring(i))
					if(f) then
						local fu = f:GetAttribute("unit")
						if(UnitIsUnit(fu, unit)) then
							UnitFrames[unit] = f
							break
						end
					end
				end
			end
		end
	end
	return UnitFrames[unit]
end

local backdrop = {
	bgFile = "Interface\\Buttons\\WHITE8X8",
	edgeFile = "Interface\\Buttons\\WHITE8X8",
	tile = false,
	tileSize = 0,
	edgeSize = 2,
	insets = { left=0, right=0, top=0, bottom=0 },
}

function SRFO:CreateTestFrame(name)
	TestFrames[#TestFrames+1] = CreateFrame("Frame", "SRFO" .. name, UIParent)
	TestFrames[#TestFrames]:SetFrameStrata("HIGH")
	TestFrames[#TestFrames]:SetWidth(200)
	TestFrames[#TestFrames]:SetHeight(100)
	TestFrames[#TestFrames]:SetPoint("RIGHT", UIParent, "RIGHT", 0.0, -100.0*(#TestFrames-1))
	TestFrames[#TestFrames]:SetBackdrop(backdrop)
	TestFrames[#TestFrames]:SetBackdropColor(.3, .3, .3, 1)
	TestFrames[#TestFrames]:SetBackdropBorderColor(.5, .5, .5, 1)
	local tex = TestFrames[#TestFrames]:CreateTexture("Texture")
	tex:SetAllPoints()
	tex:SetTexture(.3, .3, .3)
	tex:SetAlpha(0.0)
	local font = TestFrames[#TestFrames]:CreateFontString(name .. "Label", nil, "GameFontNormalSmall")
	font:SetAllPoints(TestFrames[#TestFrames])
	font:SetText(L[name])
end

SRFO:CreateTestFrame("TestPartyFrame")
SRFO:CreateTestFrame("TestPartyPetFrame")
SRFO:CreateTestFrame("TestRaidFrame")
SRFO:CreateTestFrame("TestRaidPetFrame")
